@@ -30,6 +30,8 @@ email                : sherman at mrcc.com
 #include "qgsattributes.h"
 #include "qgsfields.h"
 #include "qgsfeatureid.h"
+#include "qgsvariantutils.h"
+
 #include <memory>
 class QgsFeature;
 class QgsFeaturePrivate;
@@ -69,29 +71,88 @@ class CORE_EXPORT QgsFeature
   public:
 
 #ifdef SIP_RUN
-    SIP_PYOBJECT __iter__();
+    SIP_PYOBJECT __iter__() SIP_HOLDGIL;
     % MethodCode
     QgsAttributes attributes = sipCpp->attributes();
     PyObject *attrs = sipConvertFromType( &attributes, sipType_QgsAttributes, Py_None );
     sipRes = PyObject_GetIter( attrs );
     % End
+#endif
 
-    SIP_PYOBJECT __getitem__( int key );
+#ifdef SIP_PYQT5_RUN
+#ifdef SIP_RUN
+    SIP_PYOBJECT __getitem__( int key ) SIP_HOLDGIL;
     % MethodCode
-    QgsAttributes attrs = sipCpp->attributes();
-    if ( a0 < 0 || a0 >= attrs.count() )
+    if ( a0 < 0 || a0 >= sipCpp->attributeCount() )
     {
       PyErr_SetString( PyExc_KeyError, QByteArray::number( a0 ) );
       sipIsErr = 1;
     }
     else
     {
-      QVariant *v = new QVariant( attrs.at( a0 ) );
-      sipRes = sipConvertFromNewType( v, sipType_QVariant, Py_None );
+      const QVariant v = sipCpp->attribute( a0 );
+      if ( !v.isValid() )
+      {
+        Py_INCREF( Py_None );
+        sipRes = Py_None;
+      }
+      // QByteArray null handling is "special"! See null_from_qvariant_converter in conversions.sip
+      else if ( QgsVariantUtils::isNull( v, true ) && v.userType() != QMetaType::Type::QByteArray )
+      {
+        PyObject *vartype = sipConvertFromEnum( v.type(), sipType_QVariant_Type );
+        PyObject *args = PyTuple_Pack( 1, vartype );
+        PyTypeObject *typeObj = sipTypeAsPyTypeObject( sipType_QVariant );
+        sipRes = PyObject_Call( ( PyObject * )typeObj, args, nullptr );
+        Py_DECREF( args );
+        Py_DECREF( vartype );
+      }
+      else
+      {
+        switch ( v.userType() )
+        {
+          case QMetaType::Type::Int:
+            sipRes = PyLong_FromLong( v.toInt() );
+            break;
+
+          case QMetaType::Type::UInt:
+            sipRes = PyLong_FromUnsignedLong( v.toUInt() );
+            break;
+
+          case QMetaType::Type::Long:
+          case QMetaType::Type::LongLong:
+            sipRes = PyLong_FromLongLong( v.toLongLong() );
+            break;
+
+          case QMetaType::Type::ULong:
+          case QMetaType::Type::ULongLong:
+            sipRes = PyLong_FromUnsignedLongLong( v.toULongLong() );
+            break;
+
+          case QMetaType::Type::Bool:
+            sipRes = PyBool_FromLong( v.toBool() ? 1 : 0 );
+            break;
+
+          case QMetaType::Type::Float:
+          case QMetaType::Type::Double:
+            sipRes = PyFloat_FromDouble( v.toDouble() );
+            break;
+
+          case QMetaType::Type::QString:
+            sipRes = PyUnicode_FromString( v.toString().toUtf8().constData() );
+            break;
+
+          default:
+          {
+            QVariant *newV = new QVariant( v );
+            sipRes = sipConvertFromNewType( newV, sipType_QVariant, Py_None );
+            break;
+          }
+        }
+      }
     }
     % End
 
-    SIP_PYOBJECT __getitem__( const QString &name );
+    SIP_PYOBJECT __getitem__( const QString &name ) SIP_HOLDGIL;
     % MethodCode
     int fieldIdx = sipCpp->fieldNameIndex( *a0 );
     if ( fieldIdx == -1 )
@@ -101,22 +162,241 @@ class CORE_EXPORT QgsFeature
     }
     else
     {
-      QVariant *v = new QVariant( sipCpp->attribute( fieldIdx ) );
-      sipRes = sipConvertFromNewType( v, sipType_QVariant, Py_None );
+      const QVariant v = sipCpp->attribute( fieldIdx );
+      if ( !v.isValid() )
+      {
+        Py_INCREF( Py_None );
+        sipRes = Py_None;
+      }
+      // QByteArray null handling is "special"! See null_from_qvariant_converter in conversions.sip
+      else if ( QgsVariantUtils::isNull( v, true ) && v.userType() != QMetaType::Type::QByteArray )
+      {
+        PyObject *vartype = sipConvertFromEnum( v.type(), sipType_QVariant_Type );
+        PyObject *args = PyTuple_Pack( 1, vartype );
+        PyTypeObject *typeObj = sipTypeAsPyTypeObject( sipType_QVariant );
+        sipRes = PyObject_Call( ( PyObject * )typeObj, args, nullptr );
+        Py_DECREF( args );
+        Py_DECREF( vartype );
+      }
+      else
+      {
+        switch ( v.userType() )
+        {
+          case QMetaType::Type::Int:
+            sipRes = PyLong_FromLong( v.toInt() );
+            break;
+
+          case QMetaType::Type::UInt:
+            sipRes = PyLong_FromUnsignedLong( v.toUInt() );
+            break;
+
+          case QMetaType::Type::Long:
+          case QMetaType::Type::LongLong:
+            sipRes = PyLong_FromLongLong( v.toLongLong() );
+            break;
+
+          case QMetaType::Type::ULong:
+          case QMetaType::Type::ULongLong:
+            sipRes = PyLong_FromUnsignedLongLong( v.toULongLong() );
+            break;
+
+          case QMetaType::Type::Bool:
+            sipRes = PyBool_FromLong( v.toBool() ? 1 : 0 );
+            break;
+
+          case QMetaType::Type::Float:
+          case QMetaType::Type::Double:
+            sipRes = PyFloat_FromDouble( v.toDouble() );
+            break;
+
+          case QMetaType::Type::QString:
+            sipRes = PyUnicode_FromString( v.toString().toUtf8().constData() );
+            break;
+
+          default:
+          {
+            QVariant *newV = new QVariant( v );
+            sipRes = sipConvertFromNewType( newV, sipType_QVariant, Py_None );
+            break;
+          }
+        }
+      }
     }
     % End
+#endif
+#endif
 
-    void __setitem__( int key, QVariant value / GetWrapper / );
+#ifdef SIP_PYQT6_RUN
+#ifdef SIP_RUN
+    SIP_PYOBJECT __getitem__( int key ) SIP_HOLDGIL;
     % MethodCode
-    bool rv;
-
-    if ( a1Wrapper == Py_None )
+    if ( a0 < 0 || a0 >= sipCpp->attributeCount() )
     {
-      rv = sipCpp->setAttribute( a0, QVariant( QVariant::Int ) );
+      PyErr_SetString( PyExc_KeyError, QByteArray::number( a0 ) );
+      sipIsErr = 1;
     }
     else
     {
-      rv = sipCpp->setAttribute( a0, *a1 );
+      const QVariant v = sipCpp->attribute( a0 );
+      // QByteArray null handling is "special"! See null_from_qvariant_converter in conversions.sip
+      if ( QgsVariantUtils::isNull( v, true ) && v.userType() != QMetaType::Type::QByteArray )
+      {
+        Py_INCREF( Py_None );
+        sipRes = Py_None;
+      }
+      else
+      {
+        switch ( v.userType() )
+        {
+          case QMetaType::Type::Int:
+            sipRes = PyLong_FromLong( v.toInt() );
+            break;
+
+          case QMetaType::Type::UInt:
+            sipRes = PyLong_FromUnsignedLong( v.toUInt() );
+            break;
+
+          case QMetaType::Type::Long:
+          case QMetaType::Type::LongLong:
+            sipRes = PyLong_FromLongLong( v.toLongLong() );
+            break;
+
+          case QMetaType::Type::ULong:
+          case QMetaType::Type::ULongLong:
+            sipRes = PyLong_FromUnsignedLongLong( v.toULongLong() );
+            break;
+
+          case QMetaType::Type::Bool:
+            sipRes = PyBool_FromLong( v.toBool() ? 1 : 0 );
+            break;
+
+          case QMetaType::Type::Float:
+          case QMetaType::Type::Double:
+            sipRes = PyFloat_FromDouble( v.toDouble() );
+            break;
+
+          case QMetaType::Type::QString:
+            sipRes = PyUnicode_FromString( v.toString().toUtf8().constData() );
+            break;
+
+          default:
+          {
+            QVariant *newV = new QVariant( v );
+            sipRes = sipConvertFromNewType( newV, sipType_QVariant, Py_None );
+            break;
+          }
+        }
+      }
+    }
+    % End
+
+    SIP_PYOBJECT __getitem__( const QString &name ) SIP_HOLDGIL;
+    % MethodCode
+    int fieldIdx = sipCpp->fieldNameIndex( *a0 );
+    if ( fieldIdx == -1 )
+    {
+      PyErr_SetString( PyExc_KeyError, a0->toLatin1() );
+      sipIsErr = 1;
+    }
+    else
+    {
+      const QVariant v = sipCpp->attribute( fieldIdx );
+      // QByteArray null handling is "special"! See null_from_qvariant_converter in conversions.sip
+      if ( QgsVariantUtils::isNull( v, true ) && v.userType() != QMetaType::Type::QByteArray )
+      {
+        Py_INCREF( Py_None );
+        sipRes = Py_None;
+      }
+      else
+      {
+        switch ( v.userType() )
+        {
+          case QMetaType::Type::Int:
+            sipRes = PyLong_FromLong( v.toInt() );
+            break;
+
+          case QMetaType::Type::UInt:
+            sipRes = PyLong_FromUnsignedLong( v.toUInt() );
+            break;
+
+          case QMetaType::Type::Long:
+          case QMetaType::Type::LongLong:
+            sipRes = PyLong_FromLongLong( v.toLongLong() );
+            break;
+
+          case QMetaType::Type::ULong:
+          case QMetaType::Type::ULongLong:
+            sipRes = PyLong_FromUnsignedLongLong( v.toULongLong() );
+            break;
+
+          case QMetaType::Type::Bool:
+            sipRes = PyBool_FromLong( v.toBool() ? 1 : 0 );
+            break;
+
+          case QMetaType::Type::Float:
+          case QMetaType::Type::Double:
+            sipRes = PyFloat_FromDouble( v.toDouble() );
+            break;
+
+          case QMetaType::Type::QString:
+            sipRes = PyUnicode_FromString( v.toString().toUtf8().constData() );
+            break;
+
+          default:
+          {
+            QVariant *newV = new QVariant( v );
+            sipRes = sipConvertFromNewType( newV, sipType_QVariant, Py_None );
+            break;
+          }
+        }
+      }
+    }
+    % End
+#endif
+#endif
+
+#ifdef SIP_RUN
+    void __setitem__( int key, SIP_PYOBJECT value SIP_TYPEHINT( Optional[Union[bool, int, float, str, QVariant]] ) ) SIP_HOLDGIL;
+    % MethodCode
+    bool rv;
+
+    if ( a1 == Py_None )
+    {
+      rv = sipCpp->setAttribute( a0, QVariant( QVariant::Int ) );
+    }
+    else if ( PyBool_Check( a1 ) )
+    {
+      rv = sipCpp->setAttribute( a0, QVariant( PyObject_IsTrue( a1 ) == 1 ) );
+    }
+    else if ( PyLong_Check( a1 ) )
+    {
+      rv = sipCpp->setAttribute( a0, QVariant( PyLong_AsLongLong( a1 ) ) );
+    }
+    else if ( PyFloat_Check( a1 ) )
+    {
+      rv = sipCpp->setAttribute( a0, QVariant( PyFloat_AsDouble( a1 ) ) );
+    }
+    else if ( PyUnicode_Check( a1 ) )
+    {
+      rv = sipCpp->setAttribute( a0, QVariant( QString::fromUtf8( PyUnicode_AsUTF8( a1 ) ) ) );
+    }
+    else if ( sipCanConvertToType( a1, sipType_QVariant, SIP_NOT_NONE ) )
+    {
+      int state;
+      QVariant *qvariant = reinterpret_cast<QVariant *>( sipConvertToType( a1, sipType_QVariant, 0, SIP_NOT_NONE, &state, &sipIsErr ) );
+      if ( sipIsErr )
+      {
+        rv = false;
+      }
+      else
+      {
+        rv = sipCpp->setAttribute( a0, *qvariant );
+      }
+      sipReleaseType( qvariant, sipType_QVariant, state );
+    }
+    else
+    {
+      rv = false;
     }
 
     if ( !rv )
@@ -126,7 +406,7 @@ class CORE_EXPORT QgsFeature
     }
     % End
 
-    void __setitem__( const QString &key, QVariant value / GetWrapper / );
+    void __setitem__( const QString &key, SIP_PYOBJECT value SIP_TYPEHINT( Optional[Union[bool, int, float, str, QVariant]] ) ) SIP_HOLDGIL;
     % MethodCode
     int fieldIdx = sipCpp->fieldNameIndex( *a0 );
     if ( fieldIdx == -1 )
@@ -136,20 +416,46 @@ class CORE_EXPORT QgsFeature
     }
     else
     {
-      if ( a1Wrapper == Py_None )
+      if ( a1 == Py_None )
       {
-        sipCpp->setAttribute( *a0, QVariant( QVariant::Int ) );
+        sipCpp->setAttribute( fieldIdx, QVariant( QVariant::Int ) );
+      }
+      else if ( PyBool_Check( a1 ) )
+      {
+        sipCpp->setAttribute( fieldIdx, QVariant( PyObject_IsTrue( a1 ) == 1 ) );
+      }
+      else if ( PyLong_Check( a1 ) )
+      {
+        sipCpp->setAttribute( fieldIdx, QVariant( PyLong_AsLongLong( a1 ) ) );
+      }
+      else if ( PyFloat_Check( a1 ) )
+      {
+        sipCpp->setAttribute( fieldIdx, QVariant( PyFloat_AsDouble( a1 ) ) );
+      }
+      else if ( PyUnicode_Check( a1 ) )
+      {
+        sipCpp->setAttribute( fieldIdx, QVariant( QString::fromUtf8( PyUnicode_AsUTF8( a1 ) ) ) );
+      }
+      else if ( sipCanConvertToType( a1, sipType_QVariant, SIP_NOT_NONE ) )
+      {
+        int state;
+        QVariant *qvariant = reinterpret_cast<QVariant *>( sipConvertToType( a1, sipType_QVariant, 0, SIP_NOT_NONE, &state, &sipIsErr ) );
+        if ( !sipIsErr )
+        {
+          sipCpp->setAttribute( fieldIdx, *qvariant );
+        }
+        sipReleaseType( qvariant, sipType_QVariant, state );
       }
       else
       {
-        sipCpp->setAttribute( fieldIdx, *a1 );
+        sipIsErr = 1;
       }
     }
     % End
 
-    void __delitem__( int key );
+    void __delitem__( int key ) SIP_HOLDGIL;
     % MethodCode
-    if ( a0 >= 0 && a0 < sipCpp->attributes().count() )
+    if ( a0 >= 0 && a0 < sipCpp->attributeCount() )
       sipCpp->deleteAttribute( a0 );
     else
     {
@@ -158,7 +464,7 @@ class CORE_EXPORT QgsFeature
     }
     % End
 
-    void __delitem__( const QString &name );
+    void __delitem__( const QString &name ) SIP_HOLDGIL;
     % MethodCode
     int fieldIdx = sipCpp->fieldNameIndex( *a0 );
     if ( fieldIdx == -1 )
@@ -169,6 +475,11 @@ class CORE_EXPORT QgsFeature
     else
       sipCpp->deleteAttribute( fieldIdx );
     % End
+
+    long __hash__() const SIP_HOLDGIL;
+    % MethodCode
+    sipRes = qHash( *sipCpp );
+    % End
 #endif
 
     /**
@@ -178,7 +489,7 @@ class CORE_EXPORT QgsFeature
 #ifndef SIP_RUN
     QgsFeature( QgsFeatureId id = FID_NULL );
 #else
-    QgsFeature( qint64 id = FID_NULL );
+    QgsFeature( qint64 id = FID_NULL ) SIP_HOLDGIL;
 #endif
 
     /**
@@ -189,28 +500,28 @@ class CORE_EXPORT QgsFeature
 #ifndef SIP_RUN
     QgsFeature( const QgsFields &fields, QgsFeatureId id = FID_NULL );
 #else
-    QgsFeature( const QgsFields &fields, qint64 id = FID_NULL );
+    QgsFeature( const QgsFields &fields, qint64 id = FID_NULL ) SIP_HOLDGIL;
 #endif
 
     /**
      * Copy constructor
      */
-    QgsFeature( const QgsFeature &rhs );
+    QgsFeature( const QgsFeature &rhs ) SIP_HOLDGIL;
 
     /**
      * Assignment operator
      */
-    QgsFeature &operator=( const QgsFeature &rhs );
+    QgsFeature &operator=( const QgsFeature &rhs ) SIP_HOLDGIL;
 
     /**
      * Compares two features
      */
-    bool operator==( const QgsFeature &other ) const;
+    bool operator==( const QgsFeature &other ) const SIP_HOLDGIL;
 
     /**
      * Compares two features
      */
-    bool operator!=( const QgsFeature &other ) const;
+    bool operator!=( const QgsFeature &other ) const SIP_HOLDGIL;
 
     virtual ~QgsFeature();
 
@@ -218,7 +529,7 @@ class CORE_EXPORT QgsFeature
      * Returns the feature ID for this feature.
      * \see setId()
      */
-    QgsFeatureId id() const;
+    QgsFeatureId id() const SIP_HOLDGIL;
 
     /**
      * Sets the feature \a id for this feature.
@@ -228,7 +539,7 @@ class CORE_EXPORT QgsFeature
      *  This method is not designed to allow a specific feature ID to be assigned to a feature which will be added to a
      *  layer or data provider, and the results will be unpredictable
      */
-    void setId( QgsFeatureId id );
+    void setId( QgsFeatureId id ) SIP_HOLDGIL;
 
     /**
      * Returns the feature's attributes.
@@ -245,9 +556,8 @@ class CORE_EXPORT QgsFeature
      *
      * \see setAttributes()
      * \see attributeMap()
-     * \since QGIS 2.9
      */
-    QgsAttributes attributes() const;
+    QgsAttributes attributes() const SIP_HOLDGIL;
 
 #ifndef SIP_RUN
 
@@ -274,10 +584,10 @@ class CORE_EXPORT QgsFeature
      * \see setAttributes()
      * \since QGIS 3.22.2
      */
-    SIP_PYOBJECT attributeMap() const SIP_TYPEHINT( Dict[str, Optional[object]] );
+    SIP_PYOBJECT attributeMap() const SIP_HOLDGIL SIP_TYPEHINT( Dict[str, Optional[object]] );
     % MethodCode
     const int fieldSize = sipCpp->fields().size();
-    const int attributeSize = sipCpp->attributes().size();
+    const int attributeSize = sipCpp->attributeCount();
     if ( fieldSize == 0 && attributeSize != 0 )
     {
       PyErr_SetString( PyExc_ValueError, QStringLiteral( "Field definition has not been set for feature" ).toUtf8().constData() );
@@ -291,7 +601,8 @@ class CORE_EXPORT QgsFeature
     else
     {
       QVariantMap *v = new QVariantMap( sipCpp->attributeMap() );
-      sipRes = sipConvertFromNewType( v, sipType_QVariantMap, Py_None );
+      const sipTypeDef *qvariantmap_type = sipFindType( "QMap<QString,QVariant>" );
+      sipRes = sipConvertFromNewType( v, qvariantmap_type, Py_None );
     }
     % End
 #endif
@@ -300,7 +611,7 @@ class CORE_EXPORT QgsFeature
      * Returns the number of attributes attached to the feature.
      * \since QGIS 3.18
      */
-    int attributeCount() const;
+    int attributeCount() const SIP_HOLDGIL;
 
     /**
      * Sets the feature's attributes.
@@ -318,7 +629,7 @@ class CORE_EXPORT QgsFeature
      * \see setAttribute()
      * \see attributes()
      */
-    void setAttributes( const QgsAttributes &attrs );
+    void setAttributes( const QgsAttributes &attrs ) SIP_HOLDGIL;
 
 #ifndef SIP_RUN
 
@@ -332,7 +643,7 @@ class CORE_EXPORT QgsFeature
      * \returns FALSE, if the field index does not exist
      * \see setAttributes()
      */
-    bool setAttribute( int field, const QVariant &attr );
+    Q_INVOKABLE bool setAttribute( int field, const QVariant &attr );
 #else
 
     /**
@@ -359,17 +670,47 @@ class CORE_EXPORT QgsFeature
      * \throws KeyError if the field index does not exist
      * \see setAttributes()
      */
-    bool setAttribute( int field, const QVariant &attr / GetWrapper / );
+    bool setAttribute( int field, SIP_PYOBJECT attr SIP_TYPEHINT( Optional[Union[bool, int, float, str, QVariant]] ) ) SIP_HOLDGIL;
     % MethodCode
     bool rv;
 
-    if ( a1Wrapper == Py_None )
+    if ( a1 == Py_None )
     {
       rv = sipCpp->setAttribute( a0, QVariant( QVariant::Int ) );
     }
+    else if ( PyBool_Check( a1 ) )
+    {
+      rv = sipCpp->setAttribute( a0, QVariant( PyObject_IsTrue( a1 ) == 1 ) );
+    }
+    else if ( PyLong_Check( a1 ) )
+    {
+      rv = sipCpp->setAttribute( a0, QVariant( PyLong_AsLongLong( a1 ) ) );
+    }
+    else if ( PyFloat_Check( a1 ) )
+    {
+      rv = sipCpp->setAttribute( a0, QVariant( PyFloat_AsDouble( a1 ) ) );
+    }
+    else if ( PyUnicode_Check( a1 ) )
+    {
+      rv = sipCpp->setAttribute( a0, QVariant( QString::fromUtf8( PyUnicode_AsUTF8( a1 ) ) ) );
+    }
+    else if ( sipCanConvertToType( a1, sipType_QVariant, SIP_NOT_NONE ) )
+    {
+      int state;
+      QVariant *qvariant = reinterpret_cast<QVariant *>( sipConvertToType( a1, sipType_QVariant, 0, SIP_NOT_NONE, &state, &sipIsErr ) );
+      if ( sipIsErr )
+      {
+        rv = false;
+      }
+      else
+      {
+        rv = sipCpp->setAttribute( a0, *qvariant );
+      }
+      sipReleaseType( qvariant, sipType_QVariant, state );
+    }
     else
     {
-      rv = sipCpp->setAttribute( a0, *a1 );
+      rv = false;
     }
 
     if ( !rv )
@@ -391,7 +732,7 @@ class CORE_EXPORT QgsFeature
      *
      * \see resizeAttributes()
      */
-    void initAttributes( int fieldCount );
+    void initAttributes( int fieldCount ) SIP_HOLDGIL;
 
     /**
      * Resizes the attributes attached to this feature to the given number of fields.
@@ -406,7 +747,7 @@ class CORE_EXPORT QgsFeature
      * \see padAttributes()
      * \since QGIS 3.18
      */
-    void resizeAttributes( int fieldCount );
+    void resizeAttributes( int fieldCount ) SIP_HOLDGIL;
 
     /**
      * Resizes the attributes attached to this feature by appending the specified \a count of NULL values to the end of the existing attributes.
@@ -414,7 +755,7 @@ class CORE_EXPORT QgsFeature
      * \see resizeAttributes()
      * \since QGIS 3.18
      */
-    void padAttributes( int count );
+    void padAttributes( int count ) SIP_HOLDGIL;
 
 #ifndef SIP_RUN
 
@@ -452,9 +793,9 @@ class CORE_EXPORT QgsFeature
      * \throws KeyError if the field is not found
      * \see setAttribute()
      */
-    void deleteAttribute( int field );
+    void deleteAttribute( int field ) SIP_HOLDGIL;
     % MethodCode
-    if ( a0 >= 0 && a0 < sipCpp->attributes().count() )
+    if ( a0 >= 0 && a0 < sipCpp->attributeCount() )
       sipCpp->deleteAttribute( a0 );
     else
     {
@@ -472,7 +813,7 @@ class CORE_EXPORT QgsFeature
      *
      * \see setValid()
      */
-    bool isValid() const;
+    bool isValid() const SIP_HOLDGIL;
 
     /**
      * Sets the validity of the feature.
@@ -481,14 +822,13 @@ class CORE_EXPORT QgsFeature
      *
      * \see isValid()
      */
-    void setValid( bool validity );
+    void setValid( bool validity ) SIP_HOLDGIL;
 
     /**
      * Returns TRUE if the feature has an associated geometry.
      * \see geometry()
-     * \since QGIS 3.0.
      */
-    bool hasGeometry() const;
+    bool hasGeometry() const SIP_HOLDGIL;
 
     /**
      * Returns the geometry associated with this feature. If the feature has no geometry,
@@ -496,7 +836,7 @@ class CORE_EXPORT QgsFeature
      * \see hasGeometry()
      * \see setGeometry()
      */
-    QgsGeometry geometry() const;
+    QgsGeometry geometry() const SIP_HOLDGIL;
 
     /**
      * Set the feature's geometry.
@@ -507,7 +847,7 @@ class CORE_EXPORT QgsFeature
      * \see geometry()
      * \see clearGeometry()
      */
-    void setGeometry( const QgsGeometry &geometry );
+    void setGeometry( const QgsGeometry &geometry ) SIP_HOLDGIL;
 
     /**
      * Set the feature's \a geometry.
@@ -542,7 +882,7 @@ class CORE_EXPORT QgsFeature
 #ifndef SIP_RUN
     void setGeometry( std::unique_ptr< QgsAbstractGeometry > geometry );
 #else
-    void setGeometry( QgsAbstractGeometry *geometry SIP_TRANSFER );
+    void setGeometry( QgsAbstractGeometry *geometry SIP_TRANSFER ) SIP_HOLDGIL;
     % MethodCode
     sipCpp->setGeometry( std::unique_ptr< QgsAbstractGeometry>( a0 ) );
     % End
@@ -552,24 +892,22 @@ class CORE_EXPORT QgsFeature
      * Removes any geometry associated with the feature.
      * \see setGeometry()
      * \see hasGeometry()
-     * \since QGIS 3.0
      */
-    void clearGeometry();
+    void clearGeometry() SIP_HOLDGIL;
 
     /**
      * Assigns a field map with the feature to allow attribute access by attribute name.
      * \param fields The attribute fields which this feature holds
      * \param initAttributes If TRUE, attributes are initialized. Clears any data previously assigned.
      * \see fields()
-     * \since QGIS 2.9
      */
-    void setFields( const QgsFields &fields, bool initAttributes = false SIP_PYARGDEFAULT( true ) );
+    void setFields( const QgsFields &fields, bool initAttributes = false SIP_PYARGDEFAULT( true ) ) SIP_HOLDGIL;
 
     /**
      * Returns the field map associated with the feature.
      * \see setFields()
      */
-    QgsFields fields() const;
+    QgsFields fields() const SIP_HOLDGIL;
 
 #ifndef SIP_RUN
 
@@ -587,14 +925,12 @@ class CORE_EXPORT QgsFeature
      * \returns FALSE if attribute name could not be converted to index
      * \see setFields()
      */
-    bool setAttribute( const QString &name, const QVariant &value );
+    Q_INVOKABLE bool setAttribute( const QString &name, const QVariant &value );
 #else
 
     /**
      * Insert a value into attribute, by field \a name.
      *
-     * Returns FALSE if field \a name could not be matched.
-     *
      * Field map must be associated using setFields() before this method can be used.
      *
      * Calling this method will automatically set the feature as valid (see isValid()).
@@ -615,10 +951,10 @@ class CORE_EXPORT QgsFeature
      *
      * \param name The name of the field to set
      * \param value The value to set
-     *  \throws KeyError if the attribute name could not be converted to an index
+     * \throws KeyError if the attribute name could not could not be matched.
      * \see setFields()
      */
-    void setAttribute( const QString &name, const QVariant &value / GetWrapper / );
+    void setAttribute( const QString &name, SIP_PYOBJECT value SIP_TYPEHINT( Optional[Union[bool, int, float, str, QVariant]] ) ) SIP_HOLDGIL;
     % MethodCode
     int fieldIdx = sipCpp->fieldNameIndex( *a0 );
     if ( fieldIdx == -1 )
@@ -628,13 +964,39 @@ class CORE_EXPORT QgsFeature
     }
     else
     {
-      if ( a1Wrapper == Py_None )
+      if ( a1 == Py_None )
       {
-        sipCpp->setAttribute( *a0, QVariant( QVariant::Int ) );
+        sipCpp->setAttribute( fieldIdx, QVariant( QVariant::Int ) );
+      }
+      else if ( PyBool_Check( a1 ) )
+      {
+        sipCpp->setAttribute( fieldIdx, QVariant( PyObject_IsTrue( a1 ) == 1 ) );
+      }
+      else if ( PyLong_Check( a1 ) )
+      {
+        sipCpp->setAttribute( fieldIdx, QVariant( PyLong_AsLongLong( a1 ) ) );
+      }
+      else if ( PyFloat_Check( a1 ) )
+      {
+        sipCpp->setAttribute( fieldIdx, QVariant( PyFloat_AsDouble( a1 ) ) );
+      }
+      else if ( PyUnicode_Check( a1 ) )
+      {
+        sipCpp->setAttribute( fieldIdx, QVariant( QString::fromUtf8( PyUnicode_AsUTF8( a1 ) ) ) );
+      }
+      else if ( sipCanConvertToType( a1, sipType_QVariant, SIP_NOT_NONE ) )
+      {
+        int state;
+        QVariant *qvariant = reinterpret_cast<QVariant *>( sipConvertToType( a1, sipType_QVariant, 0, SIP_NOT_NONE, &state, &sipIsErr ) );
+        if ( !sipIsErr )
+        {
+          sipCpp->setAttribute( fieldIdx, *qvariant );
+        }
+        sipReleaseType( qvariant, sipType_QVariant, state );
       }
       else
       {
-        sipCpp->setAttribute( fieldIdx, *a1 );
+        sipIsErr = 1;
       }
     }
     % End
@@ -680,10 +1042,10 @@ class CORE_EXPORT QgsFeature
      * \endcode
      *
      * \param name The name of the field to clear
-     * \throws KeyError if attribute name could not be converted to index
+     * \throws KeyError if attribute name could not be matched.
      * \see setFields()
      */
-    bool deleteAttribute( const QString &name );
+    bool deleteAttribute( const QString &name ) SIP_HOLDGIL;
     % MethodCode
     int fieldIdx = sipCpp->fieldNameIndex( *a0 );
     if ( fieldIdx == -1 )
@@ -711,7 +1073,7 @@ class CORE_EXPORT QgsFeature
      * \returns The value of the attribute, or an invalid/null variant if no such name exists
      * \see setFields
      */
-    QVariant attribute( const QString &name ) const;
+    Q_INVOKABLE QVariant attribute( const QString &name ) const;
 #else
 
     /**
@@ -739,7 +1101,7 @@ class CORE_EXPORT QgsFeature
      * \throws KeyError if the field is not found
      * \see setFields
      */
-    SIP_PYOBJECT attribute( const QString &name ) const;
+    Q_INVOKABLE SIP_PYOBJECT attribute( const QString &name ) const SIP_HOLDGIL;
     % MethodCode
     int fieldIdx = sipCpp->fieldNameIndex( *a0 );
     if ( fieldIdx == -1 )
@@ -791,10 +1153,10 @@ class CORE_EXPORT QgsFeature
      * \throws KeyError if the field is not found
      * \see setFields()
      */
-    SIP_PYOBJECT attribute( int fieldIdx ) const;
+    SIP_PYOBJECT attribute( int fieldIdx ) const SIP_HOLDGIL;
     % MethodCode
     {
-      if ( a0 < 0 || a0 >= sipCpp->attributes().count() )
+      if ( a0 < 0 || a0 >= sipCpp->attributeCount() )
       {
         PyErr_SetString( PyExc_KeyError, QByteArray::number( a0 ) );
         sipIsErr = 1;
@@ -808,12 +1170,47 @@ class CORE_EXPORT QgsFeature
     % End
 #endif
 
+
+#ifndef SIP_RUN
+
+    /**
+     * Returns TRUE if the attribute at the specified index is an unset value.
+     *
+     * \see QgsUnsetAttributeValue
+     * \since QGIS 3.28
+     */
+    bool isUnsetValue( int fieldIdx ) const;
+#else
+
+    /**
+     * Returns TRUE if the attribute at the specified index is an unset value.
+     *
+     * \throws KeyError if the field is not found
+     * \see QgsUnsetAttributeValue
+     * \since QGIS 3.28
+     */
+    bool isUnsetValue( int fieldIdx ) const SIP_HOLDGIL;
+    % MethodCode
+    {
+      if ( a0 < 0 || a0 >= sipCpp->attributeCount() )
+      {
+        PyErr_SetString( PyExc_KeyError, QByteArray::number( a0 ) );
+        sipIsErr = 1;
+      }
+      else
+      {
+        sipRes = sipCpp->isUnsetValue( a0 );
+      }
+    }
+    % End
+#endif
+
     /**
      * Returns the feature's embedded symbology, or NULLPTR if the feature has no embedded symbol.
      *
      * \since QGIS 3.20
      */
-    const QgsSymbol *embeddedSymbol() const;
+    const QgsSymbol *embeddedSymbol() const SIP_HOLDGIL;
 
     /**
      * Sets the feature's embedded \a symbol.
@@ -822,7 +1219,7 @@ class CORE_EXPORT QgsFeature
      *
      * \since QGIS 3.20
      */
-    void setEmbeddedSymbol( QgsSymbol *symbol SIP_TRANSFER );
+    void setEmbeddedSymbol( QgsSymbol *symbol SIP_TRANSFER ) SIP_HOLDGIL;
 
     /**
      * Utility method to get attribute index from name.
@@ -833,7 +1230,7 @@ class CORE_EXPORT QgsFeature
      * \returns -1 if field does not exist or field map is not associated.
      * \see setFields()
      */
-    int fieldNameIndex( const QString &fieldName ) const;
+    int fieldNameIndex( const QString &fieldName ) const SIP_HOLDGIL;
 
     /**
      * Returns the approximate RAM usage of the feature, in bytes.
@@ -881,7 +1278,7 @@ typedef QMap<qint64, QgsGeometry> QgsGeometryMap;
 
 typedef QList<QgsFeature> QgsFeatureList;
 
-uint qHash( const QgsFeature &key, uint seed = 0 )  SIP_SKIP;
+CORE_EXPORT uint qHash( const QgsFeature &key, uint seed = 0 )  SIP_SKIP;
 
 Q_DECLARE_METATYPE( QgsFeature )
 Q_DECLARE_METATYPE( QgsFeatureList )
