@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """
 ***************************************************************************
     OneSideBuffer.py
@@ -21,10 +19,6 @@ __author__ = 'Giovanni Manghi'
 __date__ = 'January 2015'
 __copyright__ = '(C) 2015, Giovanni Manghi'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = 'f6ddc62fdb8ae589d3827e84d0c25bf669931040'
-
 from qgis.core import (QgsProcessing,
                        QgsProcessingException,
                        QgsProcessingParameterDefinition,
@@ -41,7 +35,6 @@ from processing.algs.gdal.GdalUtils import GdalUtils
 
 
 class OneSideBuffer(GdalAlgorithm):
-
     INPUT = 'INPUT'
     FIELD = 'FIELD'
     BUFFER_SIDE = 'BUFFER_SIDE'
@@ -60,7 +53,7 @@ class OneSideBuffer(GdalAlgorithm):
 
         self.addParameter(QgsProcessingParameterFeatureSource(self.INPUT,
                                                               self.tr('Input layer'),
-                                                              [QgsProcessing.TypeVectorLine]))
+                                                              [QgsProcessing.SourceType.TypeVectorLine]))
         self.addParameter(QgsProcessingParameterString(self.GEOMETRY,
                                                        self.tr('Geometry column name'),
                                                        defaultValue='geometry'))
@@ -77,7 +70,7 @@ class OneSideBuffer(GdalAlgorithm):
                                                       self.tr('Dissolve by attribute'),
                                                       None,
                                                       self.INPUT,
-                                                      QgsProcessingParameterField.Any,
+                                                      QgsProcessingParameterField.DataType.Any,
                                                       optional=True))
         self.addParameter(QgsProcessingParameterBoolean(self.DISSOLVE,
                                                         self.tr('Dissolve all results'),
@@ -90,12 +83,12 @@ class OneSideBuffer(GdalAlgorithm):
                                                      self.tr('Additional creation options'),
                                                      defaultValue='',
                                                      optional=True)
-        options_param.setFlags(options_param.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
+        options_param.setFlags(options_param.flags() | QgsProcessingParameterDefinition.Flag.FlagAdvanced)
         self.addParameter(options_param)
 
         self.addParameter(QgsProcessingParameterVectorDestination(self.OUTPUT,
                                                                   self.tr('One-sided buffer'),
-                                                                  QgsProcessing.TypeVectorPolygon))
+                                                                  QgsProcessing.SourceType.TypeVectorPolygon))
 
     def name(self):
         return 'onesidebuffer'
@@ -118,52 +111,53 @@ class OneSideBuffer(GdalAlgorithm):
             raise QgsProcessingException(self.invalidSourceError(parameters, self.INPUT))
 
         fields = source.fields()
-        ogrLayer, layerName = self.getOgrCompatibleSource(self.INPUT, parameters, context, feedback, executing)
+        input_details = self.getOgrCompatibleSource(self.INPUT, parameters, context, feedback, executing)
         geometry = self.parameterAsString(parameters, self.GEOMETRY, context)
         distance = self.parameterAsDouble(parameters, self.DISTANCE, context)
         side = self.parameterAsEnum(parameters, self.BUFFER_SIDE, context)
         fieldName = self.parameterAsString(parameters, self.FIELD, context)
-        dissolve = self.parameterAsBool(parameters, self.DISSOLVE, context)
+        dissolve = self.parameterAsBoolean(parameters, self.DISSOLVE, context)
         options = self.parameterAsString(parameters, self.OPTIONS, context)
         outFile = self.parameterAsOutputLayer(parameters, self.OUTPUT, context)
+        self.setOutputValue(self.OUTPUT, outFile)
 
-        output, outputFormat = GdalUtils.ogrConnectionStringAndFormat(outFile, context)
+        output_details = GdalUtils.gdal_connection_details_from_uri(outFile, context)
 
-        other_fields = []
-        for f in fields:
-            if f.name() == geometry:
-                continue
-            other_fields.append(f.name())
+        other_fields_exist = any(
+            True for f in fields
+            if f.name() != geometry
+        )
 
-        if other_fields:
-            other_fields = ',*'
-        else:
-            other_fields = ''
+        other_fields = ',*' if other_fields_exist else ''
 
-        arguments = []
-        arguments.append(output)
-        arguments.append(ogrLayer)
-        arguments.append('-dialect')
-        arguments.append('sqlite')
-        arguments.append('-sql')
+        arguments = [
+            output_details.connection_string,
+            input_details.connection_string,
+            '-dialect',
+            'sqlite',
+            '-sql'
+        ]
 
         if dissolve or fieldName:
-            sql = "SELECT ST_Union(ST_SingleSidedBuffer({}, {}, {})) AS {}{} FROM '{}'".format(geometry, distance, side, geometry, other_fields, layerName)
+            sql = f'SELECT ST_Union(ST_SingleSidedBuffer({geometry}, {distance}, {side})) AS {geometry}{other_fields} FROM "{input_details.layer_name}"'
         else:
-            sql = "SELECT ST_SingleSidedBuffer({}, {}, {}) AS {}{} FROM '{}'".format(geometry, distance, side, geometry, other_fields, layerName)
+            sql = f'SELECT ST_SingleSidedBuffer({geometry}, {distance}, {side}) AS {geometry}{other_fields} FROM "{input_details.layer_name}"'
 
         if fieldName:
-            sql = '"{} GROUP BY {}"'.format(sql, fieldName)
+            sql = f'{sql} GROUP BY "{fieldName}"'
 
         arguments.append(sql)
 
-        if self.parameterAsBool(parameters, self.EXPLODE_COLLECTIONS, context):
+        if self.parameterAsBoolean(parameters, self.EXPLODE_COLLECTIONS, context):
             arguments.append('-explodecollections')
 
+        if input_details.open_options:
+            arguments.extend(input_details.open_options_as_arguments())
+
         if options:
             arguments.append(options)
 
-        if outputFormat:
-            arguments.append('-f {}'.format(outputFormat))
+        if output_details.format:
+            arguments.append(f'-f {output_details.format}')
 
         return ['ogr2ogr', GdalUtils.escapeAndJoin(arguments)]
